package tech.sbdevelopment.v10lift;

import lombok.Getter;
import org.bstats.bukkit.Metrics;
import org.bstats.charts.SingleLineChart;
import org.bukkit.Bukkit;
import org.bukkit.plugin.java.JavaPlugin;
import tech.sbdevelopment.v10lift.api.V10LiftAPI;
import tech.sbdevelopment.v10lift.listeners.BlockBreakListener;
import tech.sbdevelopment.v10lift.listeners.EntityDamageListener;
import tech.sbdevelopment.v10lift.listeners.PlayerInteractListener;
import tech.sbdevelopment.v10lift.listeners.SignChangeListener;
import tech.sbdevelopment.v10lift.managers.*;
import tech.sbdevelopment.v10lift.sbutils.UpdateManager;
import tech.sbdevelopment.v10lift.sbutils.YamlFile;

import java.io.IOException;
import java.sql.SQLException;
import java.util.logging.Level;

public class V10LiftPlugin extends JavaPlugin {
    @Getter
    private static V10LiftPlugin instance;
    @Getter
    private static V10LiftAPI api;
    private static YamlFile config;
    private static DBManager dbManager;
    @Getter
    private static YamlFile messages;
    @Getter
    private static YamlFile items;
    private static boolean vault = false;
    private static boolean worldEdit = false;

    @Override
    public void onEnable() {
        instance = this;

        getLogger().info("-------------------------------");
        getLogger().info("V10Lift v" + getDescription().getVersion());
        getLogger().info("Made by SBDeveloper");
        getLogger().info(" ");

        //Load the API
        getLogger().info("Loading API...");
        api = new V10LiftAPI();

        //Load the config
        getLogger().info("Loading configs...");
        config = new YamlFile("config");
        config.loadDefaults();

        //Load the messages
        messages = new YamlFile("messages");
        messages.loadDefaults();

        //Load the items
        items = new YamlFile("items");
        items.loadDefaults();
        AntiCopyBlockManager.init();
        ForbiddenBlockManager.init();

        //Load the database
        getLogger().info("Loading lifts from storage...");
        dbManager = new DBManager(this, "data");
        try {
            dbManager.load();
        } catch (Exception e) {
            getLogger().log(Level.SEVERE, "Couldn't load lifts from data storage.", e);
        }

        //Load vault if found
        if (VaultManager.setupPermissions()) {
            getLogger().info("Loading Vault hook for group whitelist support.");
            vault = true;
        }

        //Load worldedit if found
        if (Bukkit.getPluginManager().getPlugin("WorldEdit") != null) {
            getLogger().info("Loading WorldEdit hook for selection support.");
            worldEdit = true;
        }

        //Load the command
        getLogger().info("Registering commands...");
        try {
            ACFHandler.init(this);
        } catch (IOException e) {
            getLogger().log(Level.SEVERE, "Couldn't load commands.", e);
        }

        //Register the listeners
        getLogger().info("Registering events...");
        Bukkit.getPluginManager().registerEvents(new PlayerInteractListener(), this);
        Bukkit.getPluginManager().registerEvents(new BlockBreakListener(), this);
        Bukkit.getPluginManager().registerEvents(new SignChangeListener(), this);
        Bukkit.getPluginManager().registerEvents(new EntityDamageListener(), this);

        //Load metrics
        getLogger().info("Loading metrics. Can be disabled in the global bStats config.");
        Metrics metrics = new Metrics(this, 6564);
        metrics.addCustomChart(new SingleLineChart("lifts", () -> api.getLifts().size()));

        //Load the update checker
        if (getSConfig().getFile().getBoolean("UpdateChecker.Enabled")) {
            UpdateManager updateManager = new UpdateManager(this, UpdateManager.CheckType.SPIGOT);

            updateManager.handleResponse((versionResponse, version) -> {
                switch (versionResponse) {
                    case FOUND_NEW:
                        getLogger().warning("There is a new version available! Current: " + this.getDescription().getVersion() + " New: " + version.get());
                        if (getSConfig().getFile().getBoolean("UpdateChecker.DownloadOnUpdate")) {
                            getLogger().info("Trying to download the update. This could take some time...");

                            updateManager.handleDownloadResponse((downloadResponse, fileName) -> {
                                switch (downloadResponse) {
                                    case DONE:
                                        getLogger().info("Update downloaded! If you restart your server, it will be loaded. Filename: " + fileName);
                                        break;
                                    case ERROR:
                                        getLogger().severe("Something went wrong when trying downloading the latest version.");
                                        break;
                                    case UNAVAILABLE:
                                        getLogger().warning("Unable to download the latest version.");
                                        break;
                                }
                            }).runUpdate();
                        }
                        break;
                    case LATEST:
                        getLogger().info("You are running the latest version [" + this.getDescription().getVersion() + "]!");
                        break;
                    case THIS_NEWER:
                        getLogger().info("You are running a newer version [" + this.getDescription().getVersion() + "]! This is probably fine.");
                        break;
                    case UNAVAILABLE:
                        getLogger().severe("Unable to perform an update check.");
                        break;
                }
            }).check();
        }

        getLogger().info("Plugin loaded successfully!");
        getLogger().info("-------------------------------");
    }

    @Override
    public void onDisable() {
        getLogger().info("Saving lifts...");
        try {
            dbManager.save();
        } catch (SQLException e) {
            //We can't do anything about it here, just inform the user
            getLogger().log(Level.WARNING, "Couldn't save lifts to data storage.", e);
        }
        dbManager.closeConnection();

        getLogger().info("Plugin disabled!");
        instance = null;
    }

    public static YamlFile getSConfig() {
        return config;
    }

    public static DBManager getDBManager() {
        return dbManager;
    }

    public static boolean isVaultEnabled() {
        return vault;
    }

    public static boolean isWorldEditEnabled() {
        return worldEdit;
    }
}
